C     ********************************************************************************
C
C     subroutine Routing
C     Surface and sub surface flows are propagated towards the basin outlet by
C     applying the variable parameters Muskingum-Cunge model
C
C     ********************************************************************************

      subroutine Routing(PioN)

      implicit none

      include 'dimarrays.inc'
      include 'comread.inc'
      include 'comrainrun.inc'
      include 'comvar.inc'

      integer I0,J0,IS,JS,I1,J1,K,chkok
      real QP,Acella,PioN(Nrow,Ncol)

      Acella=(DX*DY)/10**6
      QP=DX*DY/DT/1000

      do K=1,NTRA
         I0=MI0(K)  ! coordinates of the considered link
         J0=MJ0(K)
         I1=MI1(K)
         J1=MJ1(K)
         chkok=0
         do while(chkok.EQ.0)
            IS=(PUN(I0,J0)-1)/3-1
            JS=PUN(I0,J0)-5-3*IS
            IS=IS+I0
            JS=JS+J0
C           propagation of the sub surface flow
            if (AREE(I0,J0)*Acella.LT.A0) then  ! hillslope cell
C              computation of the celerity of the sub surface flow
               call Celer_Sub(I0,J0,I1,J1)
C              computation of the outflow sub surface discharge (QQI) at the time step t+dt by
C              applying the Muskingum-Cunge method 
               if (risp.GE.3) then   ! continuous rainfall-runoff simulation
                  QQI(I0,J0)=QI(I0,J0)*CI1(I0,J0)+YI(I0,J0)*CI2(I0,J0)+
     .                      YYI(I0,J0)*CI3(I0,J0)+CI4(I0,J0)*W(I0,J0)
               else
                  QQI(I0,J0)=QI(I0,J0)*CI1(I0,J0)+YI(I0,J0)*CI2(I0,J0)+
     .                       YYI(I0,J0)*CI3(I0,J0)+          ! single event simulation
     .                       CI4(I0,J0)*ES(I0,J0)*DX*DY/DT/1000.
               end if
               if (QQI(I0,J0).LT.0.) QQI(I0,J0)=0.
               QI(IS,JS)=QI(IS,JS)+QQI(I0,J0)
            else
C              cell of the river network: the sub surface discharge is added to the surface one
               Q(I0,J0)=Q(I0,J0)+QI(I0,J0)
            end if           
C           computation of the celerity of the surface flow
            call Celer_Sup(I0,J0,IS,JS)

C           computation of the outflow surface discharge (QQ) at the time step t+dt by
C           applying the Muskingum-Cunge method 
            QQ(I0,J0)=Q(I0,J0)*C1(I0,J0)+Y(I0,J0)*C2(I0,J0)+
     .                YY(I0,J0)*C3(I0,J0)+C4(I0,J0)*PioN(I0,J0)*Qp
            if (QQ(I0,J0).LT.0.) QQ(I0,J0)=0.
            Q(IS,JS)=Q(IS,JS)+QQ(I0,J0)

            if (JS.EQ.J1.AND.IS.EQ.I1) then  ! the propagation has been completed for the whole reach
               chkok=1
            else
C              one continues the propagation along the reach
               J0=JS
               I0=IS
            end if
         end do
      end do

      return
      end


C     ********************************************************************************
C
C     SUBROUTINE Celer_Sup
C     Computation of the kinematic celerity of the surface flow
C
C     ********************************************************************************

      SUBROUTINE Celer_Sup(I0,J0,I1,J1)

      include 'dimarrays.inc'
      include 'comvar.inc'
      include 'comrainrun.inc'
      include 'comread.inc'

      integer I0,J0,I1,J1
      real Bsez,ck,Sb,Wsez,Ks,Qc,Acella,DX2,zoutl,ztop
      real*8 D,Cu

      zoutl=QUO(Ichi,Jchi) ! elevation of the outlet
      ztop=maxval(QUO)     ! highest elevation of the catchment
      Acella=(DX*DY)/10**6

      if (I0.NE.I1.AND.J0.NE.J1) then
         DX2=1.4142*((DX+DY)/2)
      else
         DX2=(DX+DY)/2
      end if
      Qc=(Q(I0,J0)+Y(I0,J0)+YY(I0,J0))/3.
      if (Qc.LE.0) Qc=0.0001
      if (AREE(I0,J0).EQ.0) then
         Qc=0.0001
         AREE(I0,J0)=0.0001
      end if

      if (AREE(I0,J0)*Acella.LT.A0) then
         Wsez=Wv     ! hillslope cell
         Ks=Ksv(KsvDistr(I0,J0))
      else
         Wsez=Wr     ! cell of the river network
         Ks=Ksr1*exp((alog(Ksr0/Ksr1)/(ztop-zoutl))*(QUO(I0,J0)-zoutl))
      end if

      Sb=Slope(I0,J0)

C     computation of the kinematic celerity considering a rectangular cross section
      Bsez=((Qc*(Wsez+2.)**(2./3.)*Wsez)/(Ks*Sb**(1./2.)))**(3./8.)  ! width of the section
      ck=Ks*Sb**(1./2.)*Bsez**(2./3.)*(Wsez+2.)**(-5./3.)*
     .                                  (5./3.*Wsez+2.)
      if (ck.EQ.0) ck=0.0001

      D=Qc/(Sb*ck*Bsez*DX2)
      Cu=ck*DT/DX2             ! Courant number
      if ((Cu+D).LT.1) D=1-Cu  ! to ensure the accuracy of the routing scheme

      C1(I0,J0)=(-1+Cu+D)/(1+Cu+D)
      C2(I0,J0)=(1+Cu-D)/(1+Cu+D)
      C3(I0,J0)=(1-Cu+D)/(1+Cu+D)
      C4(I0,J0)=(2*Cu)/(1+Cu+D)

      return
      end


C     ********************************************************************************
C
C     subroutine Celer_Sub
C     Computation of the kinematic celerity of the sub surface flow
C
C     ********************************************************************************

      subroutine Celer_Sub(I0,J0,I1,J1)

      include 'dimarrays.inc'
      include 'comread.inc'
      include 'comrainrun.inc'
      include 'comvar.inc'

      integer I0,J0,I1,J1
      real ck,QcI,DX2
      real*8 Cu,D


      if (I0.NE.I1.AND.J0.NE.J1)then
         DX2=1.4142*((DX+DY)/2)
      else
         DX2=(DX+DY)/2
      end if

      QcI=(QI(I0,J0)+YI(I0,J0)+YYI(I0,J0))/3.
      if (QcI.LE.0) QcI=0.0001

C     computation of the sub surface celerity by means of the Darcy's law
      ck=Ksat*Slope(I0,J0)  

      D=2.*QcI/(Slope(I0,J0)*ck*Bp*DX2)
      Cu=ck*DT/DX2             ! Courant number
      if ((Cu+D).LT.1) D=1-Cu  ! to ensure the accuracy of the propagation scheme

      CI1(I0,J0)=(-1+Cu+D)/(1+Cu+D)
      CI2(I0,J0)=(1+Cu-D)/(1+Cu+D)
      CI3(I0,J0)=(1-Cu+D)/(1+Cu+D)
      CI4(I0,J0)=(2*Cu)/(1+Cu+D)

      return
      end